
/* This file is part of the KDE project
   SPDX-FileCopyrightText: 2016 Oleg Chernovskiy <kanedias@xaker.ru>

   SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "eventsplugin.h"

#include "events.h"

EventsPlugin::EventsPlugin(QObject *parent, const QVariantList &)
    : QObject(parent)
{
}

EventsPlugin::~EventsPlugin()
{
}
